@@ -54,7 +54,7 @@
  * @param nh              Pointer to operation handle
  * @param tag, value, ... List of tagged parameters
  *
- * @return 
+ * @return
  *    nothing
  *
  * @par Related Tags:
@@ -78,7 +78,7 @@
  * @param nh     operation handle associated with the incoming OPTIONS request
  * @param hmagic application context associated with the handle
  * @param sip    response to OPTIONS request or NULL upon an error
- *               (status code is in @a status and 
+ *               (status code is in @a status and
  *                descriptive message in @a phrase parameters)
  * @param tags   empty
  *
@@ -88,24 +88,27 @@
  */
 
 static nua_client_methods_t const nua_options_client_methods = {
-  SIP_METHOD_OPTIONS,
-  0,
-  { 
+  SIP_METHOD_OPTIONS,		/* crm_method, crm_method_name */
+  0,				/* crm_extra */
+  {				/* crm_flags */
     /* create_dialog */ 0,
     /* in_dialog */ 0,
     /* target refresh */ 0
   },
-  /*nua_options_client_template*/ NULL,
-  /*nua_options_client_init*/ NULL,
-  /*nua_options_client_request*/ NULL,
-  /* nua_options_client_check_restart */ NULL,
-  /*nua_options_client_response*/ NULL
+  NULL,				/* crm_template */
+  NULL,				/* crm_init */
+  NULL,				/* crm_send */
+  NULL,				/* crm_check_restart */
+  NULL,				/* crm_recv */
+  NULL,				/* crm_preliminary */
+  NULL,				/* crm_report */
+  NULL,				/* crm_complete */
 };
 
 int nua_stack_options(nua_t *nua,
 		      nua_handle_t *nh,
 		      nua_event_t e,
 		      tagi_t const *tags)
-{ 
+{
   return nua_client_create(nh, e, &nua_options_client_methods, tags);
 }
