@@ -77,7 +77,7 @@ struct outbound {
     unsigned gruuize:1;		/**< Establish a GRUU */
     unsigned outbound:1;	/**< Try to use outbound */
     unsigned natify:1;		/**< Try to detect NAT */
-    unsigned okeepalive:1;	/**< Connection keepalive with OPTIONS */
+    signed okeepalive:2;	/**< Connection keepalive with OPTIONS */
     unsigned validate:1;	/**< Validate registration with OPTIONS */
     /* How to detect NAT binding or connect to outbound: */
     unsigned use_connect:1;	/**< Use HTTP connect */
@@ -97,15 +97,15 @@ struct outbound {
   /** Source of Contact header */
   unsigned ob_by_stack:1;
   /** Self-generated contacts */
-  unsigned ob_contacts:1, :0;
+  unsigned ob_contacts:1;
 
   /* The registration state machine. */
   /** Initial REGISTER containing ob_rcontact has been sent */
   unsigned ob_registering:1;
-  /** 2XX response to REGISTER containg ob_rcontact has been received */
+  /** 2XX response to REGISTER containing ob_rcontact has been received */
   unsigned ob_registered:1;
-  /**The registration has been validated:
-   * We have successfully sent OPTIONS to ourselves.
+  /** The registration has been validated:
+   *  We have successfully sent OPTIONS to ourselves.
    */
   unsigned ob_validated:1;
   /** The registration has been validated once.
@@ -113,8 +113,11 @@ struct outbound {
    *   up if OPTIONS probe fails.
    */
   unsigned ob_once_validated:1;
+
+  unsigned ob_proxy_override:1;	/**< Override stack default proxy */
   unsigned :0;
 
+  url_string_t *ob_proxy;	/**< Outbound-specific proxy */
   char const *ob_instance;	/**< Our instance ID */
   int32_t ob_reg_id;		/**< Flow-id */
   sip_contact_t *ob_rcontact;	/**< Our contact */
@@ -197,7 +200,7 @@ outbound_new(outbound_owner_t *owner,
     return NULL;
 
   ob = su_home_clone((su_home_t *)owner, sizeof *ob);
-  
+
   if (ob) {
     su_md5_t md5[1];
     uint8_t digest[SU_MD5_DIGEST_SIZE];
@@ -209,10 +212,11 @@ outbound_new(outbound_owner_t *owner,
     ob->ob_nta = agent;
 
     if (instance)
-      ob->ob_instance = su_strcat_all(ob->ob_home, "+sip.instance=\"<", instance, ">\"", NULL);
+      ob->ob_instance =
+	su_sprintf(ob->ob_home, "+sip.instance=\"<%s>\"", instance);
     ob->ob_reg_id = 0;
 
-    /* Generate a cookie (used as Call-ID) for us */
+    /* Generate a random cookie (used as Call-ID) for us */
     su_md5_init(md5);
     su_guid_generate(guid);
     if (instance)
@@ -220,7 +224,7 @@ outbound_new(outbound_owner_t *owner,
     su_md5_update(md5, (void *)guid, sizeof guid);
     su_md5_digest(md5, digest);
     token64_e(ob->ob_cookie, sizeof ob->ob_cookie, digest, sizeof digest);
-    
+
     if (instance && !ob->ob_instance)
       su_home_unref(ob->ob_home), ob = NULL;
   }
@@ -246,34 +250,35 @@ void outbound_unref(outbound_t *ob)
 
 /** Set various outbound and nat-traversal related options. */
 int outbound_set_options(outbound_t *ob,
-			 char const *options,
+			 char const *_options,
 			 unsigned interval,
 			 unsigned stream_interval)
 {
   struct outbound_prefs prefs[1] = {{ 0 }};
-  char const *s;
+  char *s, *options = su_strdup(NULL, _options);
+  int invalid;
 
   prefs->interval = interval;
   prefs->stream_interval = stream_interval;
 
+#define MATCH(v) (len == sizeof(#v) - 1 && strncasecmp(#v, s, len) == 0)
+
+  if (options) {
+    for (s = options; s[0]; s++) if (s[0] == '-') s[0] = '_';
+  }
+
   prefs->gruuize = 1;
   prefs->outbound = 0;
   prefs->natify = 1;
-  prefs->okeepalive = 1;
+  prefs->okeepalive = -1;
   prefs->validate = 1;
   prefs->use_rport = 1;
 
-#define MATCH(v) (len == sizeof(#v) - 1 && strncasecmp(#v, s, len) == 0)
-
   for (s = options; s && s[0]; ) {
     size_t len = span_token(s);
     int value = 1;
 
-    if (len > 3 && strncasecmp(s, "no-", 3) == 0)
-      value = 0, s += 3, len -= 3;
-    else if (len > 4 && strncasecmp(s, "not-", 4) == 0)
-      value = 0, s += 4, len -= 4;
-    else if (len > 3 && strncasecmp(s, "no_", 3) == 0)
+    if (len > 3 && strncasecmp(s, "no_", 3) == 0)
       value = 0, s += 3, len -= 3;
     else if (len > 4 && strncasecmp(s, "not_", 4) == 0)
       value = 0, s += 4, len -= 4;
@@ -284,14 +289,12 @@ int outbound_set_options(outbound_t *ob,
     else if (MATCH(outbound)) prefs->outbound = value;
     else if (MATCH(natify)) prefs->natify = value;
     else if (MATCH(validate)) prefs->validate = value;
-    else if (MATCH(options-keepalive)) prefs->okeepalive = value;
     else if (MATCH(options_keepalive)) prefs->okeepalive = value;
-    else if (MATCH(use-connect)) prefs->use_connect = value;
     else if (MATCH(use_connect)) prefs->use_connect = value;
-    else if (MATCH(use-rport) || MATCH(use_rport)) prefs->use_rport = value;
-    else if (MATCH(use-socks) || MATCH(use_socks)) prefs->use_socks = value;
-    else if (MATCH(use-upnp) || MATCH(use_upnp)) prefs->use_upnp = value;
-    else if (MATCH(use-stun) || MATCH(use_stun)) prefs->use_stun = value;
+    else if (MATCH(use_rport)) prefs->use_rport = value;
+    else if (MATCH(use_socks)) prefs->use_socks = value;
+    else if (MATCH(use_upnp)) prefs->use_upnp = value;
+    else if (MATCH(use_stun)) prefs->use_stun = value;
     else
       SU_DEBUG_1(("outbound(%p): unknown option \"%.*s\"\n",
 		  (void *)ob->ob_owner, (int)len, s));
@@ -303,8 +306,11 @@ int outbound_set_options(outbound_t *ob,
     s += len;
   }
 
-  if (s && s[0]) {
-    SU_DEBUG_1(("outbound(%p): invalid options \"%s\"\n", 
+  invalid = s && s[0];
+  su_free(NULL, options);
+
+  if (invalid) {
+    SU_DEBUG_1(("outbound(%p): invalid options \"%s\"\n",
 		(void *)ob->ob_owner, options));
     return -1;
   }
@@ -326,11 +332,30 @@ int outbound_set_options(outbound_t *ob,
   return 0;
 }
 
+/** Override stack default proxy for outbound */
+int outbound_set_proxy(outbound_t *ob,
+		       url_string_t *proxy)
+{
+  url_string_t *new_proxy = NULL, *old_proxy = ob->ob_proxy;
+
+  if (proxy)
+    new_proxy = (url_string_t *)url_as_string(ob->ob_home, proxy->us_url);
+
+  if (proxy == NULL || new_proxy != NULL) {
+    ob->ob_proxy_override = 1;
+    ob->ob_proxy = new_proxy;
+    su_free(ob->ob_home, old_proxy);
+    return 0;
+  }
+
+  return -1;
+}
+
 /* ---------------------------------------------------------------------- */
 
 /** Obtain contacts for REGISTER */
-int outbound_get_contacts(outbound_t *ob, 
-			  sip_contact_t **return_current_contact, 
+int outbound_get_contacts(outbound_t *ob,
+			  sip_contact_t **return_current_contact,
 			  sip_contact_t **return_previous_contact)
 {
   if (ob) {
@@ -363,27 +388,31 @@ int outbound_register_response(outbound_t *ob,
   if (terminating) {
     ob->ob_registering = ob->ob_registered = 0;
     return 0;			/* Cleanup is done separately */
-  }  
+  }
 
   if (!response || !request)
     return 0;
 
   assert(request->sip_request); assert(response->sip_status);
-  
-  reregister = outbound_check_for_nat(ob, request, response);
-  if (reregister)
-    return reregister;
 
   status = response->sip_status->st_status;
 
   if (status < 300) {
-    if (request->sip_contact && response->sip_contact)
+    if (request->sip_contact && response->sip_contact) {
+      if (ob->ob_rcontact != NULL)
+        msg_header_free(ob->ob_home, (msg_header_t *)ob->ob_rcontact);
+      ob->ob_rcontact = sip_contact_dup(ob->ob_home, request->sip_contact);
       ob->ob_registered = ob->ob_registering;
-    else
+    } else
       ob->ob_registered = 0;
+  }
 
-    if (ob->ob_previous)
-      msg_header_free(ob->ob_home, (void *)ob->ob_previous);
+  reregister = outbound_check_for_nat(ob, request, response);
+  if (reregister)
+    return reregister;
+
+  if (ob->ob_previous && status < 300) {
+    msg_header_free(ob->ob_home, (void *)ob->ob_previous);
     ob->ob_previous = NULL;
   }
 
@@ -416,7 +445,7 @@ int outbound_check_for_nat(outbound_t *ob,
   if (!ob->ob_by_stack)
     return ob_no_nat;
 
-  /* Application does not want us to do any NAT traversal */ 
+  /* Application does not want us to do any NAT traversal */
   if (!ob->ob_prefs.natify)
     return ob_no_nat;
 
@@ -503,13 +532,13 @@ int outbound_nat_detect(outbound_t *ob,
   }
 
   if (!nat_detected) {
-    SU_DEBUG_1(("outbound(%p): detected NAT: %s != %s\n",
+    SU_DEBUG_5(("outbound(%p): detected NAT: %s != %s\n",
 		(void *)ob->ob_owner, v->v_host, received));
     if (ob->ob_oo && ob->ob_oo->oo_status)
       ob->ob_oo->oo_status(ob->ob_owner, ob, 101, "NAT detected", TAG_END());
   }
   else {
-    SU_DEBUG_1(("outbound(%p): NAT binding changed: "
+    SU_DEBUG_5(("outbound(%p): NAT binding changed: "
 		"[%s]:%s != [%s]:%s\n",
 		(void *)ob->ob_owner, nat_detected, nat_port, received, rport));
     if (ob->ob_oo && ob->ob_oo->oo_status)
@@ -582,7 +611,7 @@ int outbound_gruuize(outbound_t *ob, sip_t const *sip)
   }
 
   gruu = (char *)msg_header_find_param(m->m_common, "pub-gruu=");
-  
+
   if (gruu == NULL || gruu[0] == '\0')
     gruu = (char *)msg_header_find_param(m->m_common, "gruu=");
 
@@ -614,6 +643,11 @@ static int keepalive_options_with_registration_probe(outbound_t *ob);
 static int response_to_keepalive_options(outbound_t *ob,
 					 nta_outgoing_t *orq,
 					 sip_t const *sip);
+static int process_response_to_keepalive_options(outbound_t *ob,
+						 nta_outgoing_t *orq,
+						 sip_t const *sip,
+						 int status,
+						 char const *phrase);
 
 static void keepalive_timer(su_root_magic_t *root_magic,
 			    su_timer_t *t,
@@ -624,16 +658,22 @@ void outbound_start_keepalive(outbound_t *ob,
 			      nta_outgoing_t *register_transaction)
 {
   unsigned interval = 0;
-  int need_to_validate;
+  int need_to_validate, udp;
 
   if (!ob)
     return;
 
-  if (ob->ob_prefs.natify && ob->ob_prefs.okeepalive)
+  udp = ob->ob_via && ob->ob_via->v_protocol == sip_transport_udp;
+
+  if (/* ob->ob_prefs.natify && */
+      /* On UDP, use OPTIONS keepalive by default */
+      (udp ? ob->ob_prefs.okeepalive != 0
+       /* Otherwise, only if requested */
+       : ob->ob_prefs.okeepalive > 0))
     interval = ob->ob_prefs.interval;
   need_to_validate = ob->ob_prefs.validate && !ob->ob_validated;
 
-  if (!ob->ob_nat_detected || !register_transaction ||
+  if (!register_transaction ||
       !(need_to_validate || interval != 0)) {
     outbound_stop_keepalive(ob);
     return;
@@ -648,7 +688,7 @@ void outbound_start_keepalive(outbound_t *ob,
 
   ob->ob_keepalive.interval = interval;
 
-  if (!ob->ob_validated && ob->ob_keepalive.sipstun 
+  if (!ob->ob_validated && ob->ob_keepalive.sipstun
       && 0 /* Stun is disabled for now */) {
     nta_tport_keepalive(register_transaction);
   }
@@ -690,6 +730,9 @@ static int create_keepalive_message(outbound_t *ob, sip_t const *regsip)
 
   unsigned d = ob->ob_keepalive.interval;
 
+  if (msg == NULL)
+    return -1;
+
   assert(regsip); assert(regsip->sip_request);
 
   if (m && m->m_params) {
@@ -707,7 +750,7 @@ static int create_keepalive_message(outbound_t *ob, sip_t const *regsip)
       s = su_strdup(msg_home(msg), s);
       msg_header_add_param(msg_home(msg), ac->cp_common, s);
     }
-    
+
     if (features)
       msg_header_insert(msg, NULL, (void *)ac);
     else
@@ -776,6 +819,8 @@ static int keepalive_options(outbound_t *ob)
 			 ob,
 			 NULL,
 			 req,
+			 TAG_IF(ob->ob_proxy_override,
+				NTATAG_DEFAULT_PROXY(ob->ob_proxy)),
 			 TAG_END());
 
   if (!ob->ob_keepalive.orq)
@@ -790,10 +835,6 @@ static int response_to_keepalive_options(outbound_t *ob,
 {
   int status = 408;
   char const *phrase = sip_408_Request_timeout;
-  int binding_check;
-  int challenged = 0, credentials = 0;
-  msg_t *_reqmsg = nta_outgoing_getrequest(orq);
-  sip_t *request = sip_object(_reqmsg); msg_destroy(_reqmsg);
 
   if (sip && sip->sip_status) {
     status = sip->sip_status->st_status;
@@ -804,8 +845,33 @@ static int response_to_keepalive_options(outbound_t *ob,
     /* This probably means that we are in trouble. whattodo, whattodo */
   }
 
-  if (status < 200)
+  if (status >= 200) {
+    if (orq == ob->ob_keepalive.orq)
+      ob->ob_keepalive.orq = NULL;
+    process_response_to_keepalive_options(ob, orq, sip, status, phrase);
+    nta_outgoing_destroy(orq);
+  }
+
+  return 0;
+}
+
+static int process_response_to_keepalive_options(outbound_t *ob,
+						 nta_outgoing_t *orq,
+						 sip_t const *sip,
+						 int status,
+						 char const *phrase)
+{
+  int binding_check;
+  int challenged = 0, credentials = 0;
+  msg_t *_reqmsg = nta_outgoing_getrequest(orq);
+  sip_t *request = sip_object(_reqmsg); msg_destroy(_reqmsg);
+
+  if (sip == NULL) {
+    SU_DEBUG_3(("outbound(%p): keepalive %u %s\n", (void *)ob->ob_owner,
+		status, phrase));
+    ob->ob_oo->oo_keepalive_error(ob->ob_owner, ob, status, phrase, TAG_END());
     return 0;
+  }
 
   if (status == 401 || status == 407) {
     if (sip->sip_www_authenticate)
@@ -825,10 +891,6 @@ static int response_to_keepalive_options(outbound_t *ob,
 
   binding_check = outbound_nat_detect(ob, request, sip);
 
-  if (orq == ob->ob_keepalive.orq)
-    ob->ob_keepalive.orq = NULL;
-  nta_outgoing_destroy(orq);
-
   if (binding_check > 1) {
     /* Bindings have changed */
     if (outbound_contacts_from_via(ob, sip->sip_via) == 0) {
@@ -851,42 +913,40 @@ static int response_to_keepalive_options(outbound_t *ob,
 
     if (status < 300 && ob->ob_keepalive.validated) {
       loglevel = 5;
-      if (ob->ob_validated) 
+      if (ob->ob_validated)
 	loglevel = 99;		/* only once */
       ob->ob_validated = ob->ob_once_validated = 1;
     }
-    else if (status == 401 || status == 407 || status == 403) 
+    else if (status == 401 || status == 407 || status == 403)
       loglevel = 5, failed = 1;
     else
       loglevel = 3, failed = 1;
-      
-    loglevel = 1;		/* XXX ... for now */
 
     if (loglevel >= SU_LOG->log_level) {
       sip_contact_t const *m = ob->ob_rcontact;
 
       if  (m)
-	su_llog(SU_LOG, loglevel,       
+	su_llog(SU_LOG, loglevel,
 		"outbound(%p): %s <" URL_PRINT_FORMAT ">\n",
 		(void *)ob->ob_owner,
 		failed ? "FAILED to validate" : "validated",
 		URL_PRINT_ARGS(m->m_url));
       else
-	su_llog(SU_LOG, loglevel,       
+	su_llog(SU_LOG, loglevel,
 		"outbound(%p): %s registration\n",
 		(void *)ob->ob_owner,
 		failed ? "FAILED to validate" : "validated");
 
       if (failed)
-	su_llog(SU_LOG, loglevel, "outbound(%p): FAILED with %u %s\n", 
+	su_llog(SU_LOG, loglevel, "outbound(%p): FAILED with %u %s\n",
 		(void *)ob->ob_owner, status, phrase);
     }
 
-    if (failed) 
+    if (failed)
       ob->ob_oo->oo_probe_error(ob->ob_owner, ob, status, phrase, TAG_END());
   }
   else if (status == 408) {
-    SU_DEBUG_1(("outbound(%p): keepalive timeout\n", (void *)ob->ob_owner));
+    SU_DEBUG_3(("outbound(%p): keepalive timeout\n", (void *)ob->ob_owner));
     ob->ob_oo->oo_keepalive_error(ob->ob_owner, ob, status, phrase, TAG_END());
     return 0;
   }
@@ -943,6 +1003,8 @@ static int keepalive_options_with_registration_probe(outbound_t *ob)
 			 ob,
 			 NULL,
 			 req,
+			 TAG_IF(ob->ob_proxy_override,
+				NTATAG_DEFAULT_PROXY(ob->ob_proxy)),
 			 SIPTAG_SUBJECT_STR("REGISTRATION PROBE"),
 			 /* NONE is used to remove
 			    Max-Forwards: 0 found in ordinary keepalives */
@@ -961,7 +1023,7 @@ static int keepalive_options_with_registration_probe(outbound_t *ob)
 /** Check if request should be processed by outbound */
 int outbound_targeted_request(sip_t const *sip)
 {
-  return 
+  return
     sip && sip->sip_request &&
     sip->sip_request->rq_method == sip_method_options &&
     sip->sip_accept &&
@@ -970,7 +1032,7 @@ int outbound_targeted_request(sip_t const *sip)
 }
 
 /** Answer to the connectivity probe OPTIONS */
-int outbound_process_request(outbound_t *ob, 
+int outbound_process_request(outbound_t *ob,
 			     nta_incoming_t *irq,
 			     sip_t const *sip)
 {
@@ -979,7 +1041,7 @@ int outbound_process_request(outbound_t *ob,
     return 0;
 
   if (ob->ob_keepalive.validating) {
-    SU_DEBUG_1(("outbound(%p): registration check OPTIONS received\n", 
+    SU_DEBUG_5(("outbound(%p): registration check OPTIONS received\n",
 		(void *)ob->ob_owner));
     ob->ob_keepalive.validated = 1;
   }
@@ -997,7 +1059,7 @@ int outbound_process_request(outbound_t *ob,
 /**@internal
  * Create contacts for outbound.
  *
- * There are two contacts: 
+ * There are two contacts:
  * one suitable for registrations (ob_rcontact) and another that can be used
  * in dialogs (ob_dcontact).
  */
@@ -1005,7 +1067,6 @@ int outbound_contacts_from_via(outbound_t *ob, sip_via_t const *via)
 {
   su_home_t *home = ob->ob_home;
   sip_contact_t *rcontact, *dcontact;
-  int reg_id = 0;
   char reg_id_param[20] = "";
   sip_contact_t *previous_previous, *previous_rcontact, *previous_dcontact;
   sip_via_t *v, v0[1], *previous_via;
@@ -1016,16 +1077,16 @@ int outbound_contacts_from_via(outbound_t *ob, sip_via_t const *via)
 
   v = v0; *v0 = *via; v0->v_next = NULL;
 
-  dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1, 
+  dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1,
 				   v, v->v_protocol, NULL);
 
   if (ob->ob_instance && ob->ob_reg_id != 0)
     snprintf(reg_id_param, sizeof reg_id_param, ";reg-id=%u", ob->ob_reg_id);
 
   rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 0,
-				   v, v->v_protocol, 
+				   v, v->v_protocol,
 				   ob->ob_instance, reg_id_param, NULL);
-    
+
   v = sip_via_dup(home, v);
 
   if (!rcontact || !dcontact || !v) {
@@ -1044,8 +1105,10 @@ int outbound_contacts_from_via(outbound_t *ob, sip_via_t const *via)
     previous_dcontact = ob->ob_dcontact;
     previous_via = ob->ob_via;
 
-    if (ob->ob_registering &&
-        (reg_id == 0 || ob->ob_info.outbound < outbound_feature_supported))
+    if (ob->ob_registered
+        /* && (ob->ob_reg_id == 0 || ob->ob_info.outbound < outbound_feature_supported)
+         * XXX - multiple connections not yet supported
+	 */)
       previous_rcontact = NULL, ob->ob_previous = ob->ob_rcontact;
     else
       previous_rcontact = ob->ob_rcontact, ob->ob_previous = NULL;
@@ -1105,14 +1168,14 @@ int outbound_set_contact(outbound_t *ob,
   if (terminating) {
     if (ob->ob_by_stack && application_contact == NULL)
       return 0;
-    
+
     if (ob->ob_contacts)
       previous = ob->ob_rcontact;
   }
   else if (application_contact) {
     rcontact = sip_contact_dup(home, application_contact);
 
-    if (!ob->ob_rcontact || 
+    if (!ob->ob_rcontact ||
 	url_cmp_all(ob->ob_rcontact->m_url, application_contact->m_url)) {
       contact_uri_changed = 1;
       previous = ob->ob_contacts ? ob->ob_rcontact : NULL;
@@ -1122,10 +1185,10 @@ int outbound_set_contact(outbound_t *ob,
     return 0;    /* Xyzzy - nothing happens */
   }
   else if (v) {
-    char const *tport = !v->v_next ? v->v_protocol : NULL; 
+    char const *tport = !v->v_next ? v->v_protocol : NULL;
     char reg_id_param[20];
 
-    dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1, 
+    dcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 1,
 				     v, tport, NULL);
     if (!dcontact)
       return -1;
@@ -1134,18 +1197,18 @@ int outbound_set_contact(outbound_t *ob,
       snprintf(reg_id_param, sizeof reg_id_param, ";reg-id=%u", ob->ob_reg_id);
 
     rcontact = ob->ob_oo->oo_contact(ob->ob_owner, home, 0,
-				     v, v->v_protocol, 
+				     v, v->v_protocol,
 				     ob->ob_instance, reg_id_param, NULL);
     if (!rcontact)
       return -1;
 
-    if (!ob->ob_rcontact || 
+    if (!ob->ob_rcontact ||
 	url_cmp_all(ob->ob_rcontact->m_url, rcontact->m_url)) {
       contact_uri_changed = 1;
       previous = ob->ob_contacts ? ob->ob_rcontact : NULL;
     }
   }
-    
+
   ob->ob_by_stack = application_contact == NULL;
 
   ob->ob_contacts = rcontact != NULL;
