@@ -62,8 +62,8 @@ struct tp_magic_s
 /* -- Module prototypes ------------------------------------------------- */
 
 static msg_t *s2_msg(int flags);
-static int s2_complete_response(msg_t *response, 
-				int status, char const *phrase, 
+static int s2_complete_response(msg_t *response,
+				int status, char const *phrase,
 				msg_t *request);
 static char *s2_generate_tag(su_home_t *home);
 
@@ -102,6 +102,8 @@ char const s2_auth3_digest_str[] =
 
 char const s2_auth3_credentials[] = "Digest:\"s2test3\":abc:abc";
 
+int s2_nua_thread = 0;
+
 /* -- Delay scenarios --------------------------------------------------- */
 
 static unsigned long time_offset;
@@ -130,7 +132,7 @@ struct event *s2_remove_event(struct event *e)
 
   e->prev = NULL, e->next = NULL;
 
-  return e; 
+  return e;
 }
 
 void s2_free_event(struct event *e)
@@ -161,7 +163,7 @@ struct event *s2_next_event(void)
 
     su_root_step(s2->root, 100);
   }
-} 
+}
 
 struct event *s2_wait_for_event(nua_event_t event, int status)
 {
@@ -178,7 +180,7 @@ struct event *s2_wait_for_event(nua_event_t event, int status)
 
     su_root_step(s2->root, 100);
   }
-} 
+}
 
 int s2_check_event(nua_event_t event, int status)
 {
@@ -204,7 +206,7 @@ int s2_check_callstate(enum nua_callstate state)
   return retval;
 }
 
-static void 
+static void
 s2_nua_callback(nua_event_t event,
 		int status, char const *phrase,
 		nua_t *nua, nua_magic_t *_t,
@@ -216,7 +218,7 @@ s2_nua_callback(nua_event_t event,
 
   if (event == nua_i_active || event == nua_i_terminated)
     return;
-  
+
   e = calloc(1, sizeof *e);
   nua_save_event(nua, e->event);
   e->nh = nua_handle_ref(nh);
@@ -228,6 +230,8 @@ s2_nua_callback(nua_event_t event,
   *prev = e, e->prev = prev;
 }
 
+/* ---------------------------------------------------------------------- */
+/* SIP messages sent by nua */
 
 struct message *
 s2_remove_message(struct message *m)
@@ -237,7 +241,7 @@ s2_remove_message(struct message *m)
 
   m->prev = NULL, m->next = NULL;
 
-  return m; 
+  return m;
 }
 
 void
@@ -290,10 +294,10 @@ s2_wait_for_response(int status, sip_method_t method, char const *name)
 
       if (method == sip_method_unknown && name == NULL)
 	break;
-      
+
       if (m->sip->sip_cseq == NULL)
 	continue;
-      
+
       if (m->sip->sip_cseq->cs_method != method)
 	continue;
       if (name == NULL)
@@ -307,7 +311,7 @@ s2_wait_for_response(int status, sip_method_t method, char const *name)
 
     su_root_step(s2->root, 100);
   }
-} 
+}
 
 int
 s2_check_response(int status, sip_method_t method, char const *name)
@@ -331,9 +335,9 @@ s2_next_request(void)
 
     su_root_step(s2->root, 100);
   }
-  
+
   return NULL;
-} 
+}
 
 struct message *
 s2_wait_for_request(sip_method_t method, char const *name)
@@ -354,9 +358,9 @@ s2_wait_for_request(sip_method_t method, char const *name)
 
     su_root_step(s2->root, 100);
   }
-  
+
   return NULL;
-} 
+}
 
 int
 s2_check_request(sip_method_t method, char const *name)
@@ -421,7 +425,7 @@ s2_respond_to(struct message *m, struct dialog *d,
 
   *tpn = *tport_name(m->tport);
 
-  rport = su_sprintf(home, "rport=%u", 
+  rport = su_sprintf(home, "rport=%u",
 		     ntohs(((su_sockaddr_t *)
 			    msg_addrinfo(m->msg)->ai_addr)->su_port));
 
@@ -429,7 +433,7 @@ s2_respond_to(struct message *m, struct dialog *d,
       sip->sip_via->v_rport &&
       sip->sip_via->v_rport[0] == '\0') {
     msg_header_add_param(home, sip->sip_via->v_common, rport);
-  }    
+  }
 
   tpn->tpn_port = rport + strlen("rport=");
 
@@ -442,9 +446,9 @@ s2_respond_to(struct message *m, struct dialog *d,
 }
 
 /** Add headers from the request to the response message. */
-static int 
-s2_complete_response(msg_t *response, 
-		     int status, char const *phrase, 
+static int
+s2_complete_response(msg_t *response,
+		     int status, char const *phrase,
 		     msg_t *request)
 {
   su_home_t *home = msg_home(response);
@@ -465,7 +469,7 @@ s2_complete_response(msg_t *response,
   if (!response_sip->sip_to)
     response_sip->sip_to = sip_to_dup(home, request_sip->sip_to);
   if (!response_sip->sip_call_id)
-    response_sip->sip_call_id = 
+    response_sip->sip_call_id =
       sip_call_id_dup(home, request_sip->sip_call_id);
   if (!response_sip->sip_cseq)
     response_sip->sip_cseq = sip_cseq_dup(home, request_sip->sip_cseq);
@@ -492,11 +496,11 @@ s2_complete_response(msg_t *response,
   return 0;
 }
 
-/* Send request (updating dialog). 
+/* Send request (updating dialog).
  *
  * Return zero upon success, nonzero upon failure.
  */
-int 
+int
 s2_request_to(struct dialog *d,
 	      sip_method_t method, char const *name,
 	      tport_t *tport,
@@ -507,12 +511,13 @@ s2_request_to(struct dialog *d,
 
   msg_t *msg = s2_msg(0);
   sip_t *sip = sip_object(msg);
-  url_string_t *target = NULL;
+  url_t const *target = NULL;
   sip_cseq_t cseq[1];
   sip_via_t via[1]; char const *v_params[8];
   sip_content_length_t l[1];
   tp_name_t tpn[1];
   tp_magic_t *magic;
+  int user_via = 0;
 
   ta_start(ta, tag, value);
   tags = ta_args(ta);
@@ -520,26 +525,28 @@ s2_request_to(struct dialog *d,
   if (sip_add_tagis(msg, sip, &tags) < 0)
     goto error;
 
+  if (sip->sip_request)
+    target = sip->sip_request->rq_url;
+  else if (d->target)
+    target = d->target->m_url;
+  else if (s2->registration->contact)
+    target = s2->registration->contact->m_url;
+  else
+    target = NULL;
+
+  if (target == NULL)
+    goto error;
+
   if (!sip->sip_request) {
     sip_request_t *rq;
-
-    if (d->target)
-      target = (url_string_t *)d->target->m_url;
-    else if (s2->registration->contact)
-      target = (url_string_t *)s2->registration->contact->m_url;
-    else
-      target = NULL;
-
-    if (target == NULL)
-      goto error;
-
-    rq = sip_request_create(msg_home(msg), method, name, target, NULL);
+    rq = sip_request_create(msg_home(msg), method, name,
+			    (url_string_t *)target, NULL);
     sip_header_insert(msg, sip, (sip_header_t *)rq);
   }
 
   if (!d->local && sip->sip_from)
     d->local = sip_from_dup(d->home, sip->sip_from);
-  if (!d->contact && sip->sip_contact) 
+  if (!d->contact && sip->sip_contact)
     d->contact = sip_contact_dup(d->home, sip->sip_contact);
   if (!d->remote && sip->sip_to)
     d->remote = sip_to_dup(d->home, sip->sip_to);
@@ -551,7 +558,7 @@ s2_request_to(struct dialog *d,
     d->call_id = sip_call_id_dup(d->home, sip->sip_call_id);
   if (!d->lseq && sip->sip_cseq)
     d->lseq = sip->sip_cseq->cs_seq;
-  
+
   if (!d->local)
     d->local = sip_from_dup(d->home, s2->local);
   if (!d->contact)
@@ -583,11 +590,12 @@ s2_request_to(struct dialog *d,
   assert(tport);
 
   *tpn = *tport_name(tport);
-  if (tport_is_udp(tport)) {
-    tpn->tpn_host = d->target->m_url->url_host;
-    tpn->tpn_port = url_port(d->target->m_url);
+
+  if (tport_is_primary(tport)) {
+    tpn->tpn_host = target->url_host;
+    tpn->tpn_port = url_port(target);
     if (!tpn->tpn_port || !tpn->tpn_port[0])
-      tpn->tpn_port = url_port_default(d->target->m_url->url_type);
+      tpn->tpn_port = url_port_default(target->url_type);
   }
 
   magic = tport_magic(tport);
@@ -596,7 +604,7 @@ s2_request_to(struct dialog *d,
   sip_cseq_init(cseq);
   cseq->cs_method = method;
   cseq->cs_method_name = name;
-  
+
   if (d->invite && (method == sip_method_ack || method == sip_method_cancel)) {
     cseq->cs_seq = sip_object(d->invite)->sip_cseq->cs_seq;
   }
@@ -604,7 +612,10 @@ s2_request_to(struct dialog *d,
     cseq->cs_seq = ++d->lseq;
   }
 
-  if (d->invite && method == sip_method_cancel) {
+  if (sip->sip_via) {
+    user_via = 1;
+  }
+  else if (d->invite && method == sip_method_cancel) {
     *via = *sip_object(d->invite)->sip_via;
   }
   else {
@@ -618,13 +629,27 @@ s2_request_to(struct dialog *d,
   if (sip->sip_payload)
     l->l_length = sip->sip_payload->pl_len;
 
-  sip_add_tl(msg, sip, 
+  if (d->local->a_tag == NULL) {
+    char const *ltag = s2_generate_tag(d->home);
+
+    if (sip_from_tag(d->home, d->local, ltag) < 0) {
+      assert(!"add To tag");
+    }
+
+    if (sip->sip_from && sip->sip_from->a_tag == NULL) {
+      if (sip_from_tag(msg_home(msg), sip->sip_from, ltag) < 0) {
+	assert(!"add To tag");
+      }
+    }
+  }
+
+  sip_add_tl(msg, sip,
 	     TAG_IF(!sip->sip_from, SIPTAG_FROM(d->local)),
 	     TAG_IF(!sip->sip_contact, SIPTAG_CONTACT(d->contact)),
 	     TAG_IF(!sip->sip_to, SIPTAG_TO(d->remote)),
 	     TAG_IF(!sip->sip_call_id, SIPTAG_CALL_ID(d->call_id)),
 	     TAG_IF(!sip->sip_cseq, SIPTAG_CSEQ(cseq)),
-	     SIPTAG_VIA(via),
+	     TAG_IF(!user_via, SIPTAG_VIA(via)),
 	     TAG_IF(!sip->sip_content_length, SIPTAG_CONTENT_LENGTH(l)),
 	     TAG_IF(!sip->sip_separator, SIPTAG_SEPARATOR_STR("\r\n")),
 	     TAG_END());
@@ -645,7 +670,7 @@ s2_request_to(struct dialog *d,
   }
 
   return tport ? 0 : -1;
-  
+
  error:
   ta_end(ta);
   return -1;
@@ -777,14 +802,14 @@ s2_generate_tag(su_home_t *home)
 
 void s2_case(char const *number,
 	     char const *title,
-	     char const *desciption)
+	     char const *description)
 {
   _s2case = number;
 }
 
 /* ---------------------------------------------------------------------- */
 /* tport interface */
-static void 
+static void
 s2_stack_recv(struct tester *s2,
 	      tport_t *tp,
 	      msg_t *msg,
@@ -819,16 +844,16 @@ s2_stack_error(struct tester *s2,
 	       int errcode,
 	       char const *remote)
 {
-  fprintf(stderr, "%s(%p): error %d (%s) from %s\n", 
+  fprintf(stderr, "%s(%p): error %d (%s) from %s\n",
 	  "nua_tester_error",
-	  (void *)tp, errcode, su_strerror(errcode), 
+	  (void *)tp, errcode, su_strerror(errcode),
 	  remote ? remote : "<unknown destination>");
 }
 
 static msg_t *
 s2_stack_alloc(struct tester *s2, int flags,
 	       char const data[], usize_t size,
-	       tport_t const *tport, 
+	       tport_t const *tport,
 	       tp_client_t *tpc)
 {
   return msg_create(s2->mclass, flags | s2->flags);
@@ -863,11 +888,9 @@ void s2_setup_base(char const *hostname)
 
   assert(s2->root != NULL);
 
-  su_root_threading(s2->root, 0);	/* disable multithreading */
-
   s2->local = sip_from_format(s2->home, "Bob <sip:bob@%s>",
 			     hostname ? hostname : "example.net");
-  
+
   if (hostname == NULL)
     hostname = "127.0.0.1";
 
@@ -911,7 +934,9 @@ s2_setup_tport(char const * const *protocols,
   ta_start(ta, tag, value);
 
   if (s2->master == NULL) {
-    s2->master = tport_tcreate(s2, s2_stack, s2->root, ta_tags(ta));
+    s2->master = tport_tcreate(s2, s2_stack, s2->root,
+			       TPTAG_LOG(getenv("S2_TPORT_LOG") != NULL),
+			       ta_tags(ta));
 
     if (s2->master == NULL) {
       assert(s2->master);
@@ -927,8 +952,8 @@ s2_setup_tport(char const * const *protocols,
 
   if (protocols == NULL)
     protocols = default_protocols;
-  
-  bound = tport_tbind(s2->master, tpn, protocols, 
+
+  bound = tport_tbind(s2->master, tpn, protocols,
 		      TPTAG_SERVER(1),
 		      ta_tags(ta));
   assert(bound != -1);
@@ -954,7 +979,7 @@ s2_setup_tport(char const * const *protocols,
 
     v = sip_via_format(s2->home, "SIP/2.0/%s %s:%s",
 		       tpn->tpn_proto,
-		       tpn->tpn_host, 
+		       tpn->tpn_host,
 		       tpn->tpn_port);
     assert(v != NULL);
     if (strncasecmp(tpn->tpn_proto, "tls", 3)) {
@@ -963,11 +988,11 @@ s2_setup_tport(char const * const *protocols,
 			     tpn->tpn_port,
 			     tpn->tpn_proto);
       if (s2->udp.contact == NULL && strcasecmp(tpn->tpn_proto, "udp") == 0) {
-	s2->udp.tport = tport_ref(tp); 
+	s2->udp.tport = tport_ref(tp);
 	s2->udp.contact = m;
       }
       if (s2->tcp.contact == NULL && strcasecmp(tpn->tpn_proto, "tcp") == 0) {
-	s2->tcp.tport = tport_ref(tp); 
+	s2->tcp.tport = tport_ref(tp);
 	s2->tcp.contact = m;
       }
     }
@@ -982,7 +1007,7 @@ s2_setup_tport(char const * const *protocols,
 			     tpn->tpn_host,
 			     tpn->tpn_port);
       if (s2->tls.contact == NULL) {
-	s2->tls.tport = tport_ref(tp); 
+	s2->tls.tport = tport_ref(tp);
 	s2->tls.contact = m;
       }
     }
@@ -1482,16 +1507,19 @@ nua_t *s2_nua_setup(tag_type_t tag, tag_value_t value, ...)
   s2_setup_dns();
 
   s2_setup_logs(0);
-  s2_setup_tport(NULL, TPTAG_LOG(0), TAG_END());
+  s2_setup_tport(NULL, TAG_END());
   assert(s2->contact);
 
+  /* enable/disable multithreading */
+  su_root_threading(s2->root, s2_nua_thread);
+
   s2_dns_domain("example.org", 1,
 		"s2", 1, s2->udp.contact->m_url,
 		"s2", 1, s2->tcp.contact->m_url,
 		NULL);
 
   ta_start(ta, tag, value);
-  s2->nua = 
+  s2->nua =
     nua_create(s2->root,
 	       s2_nua_callback,
 	       s2,
@@ -1506,7 +1534,7 @@ nua_t *s2_nua_setup(tag_type_t tag, tag_value_t value, ...)
 #endif
 	       ta_tags(ta));
   ta_end(ta);
-  
+
   return s2->nua;
 }
 
@@ -1572,7 +1600,7 @@ void s2_register_teardown(void)
     struct message *m;
 
     nua_unregister(nh, TAG_END());
-    
+
     m = s2_wait_for_request(SIP_METHOD_REGISTER); assert(m);
     s2_save_register(m);
     s2_respond_to(m, NULL,
