@@ -23,7 +23,7 @@
  */
 
 /**@CFILE sip_pref_util.c
- * 
+ *
  * SIP callercaps and callerprefs utility functions.
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>.
@@ -33,6 +33,7 @@
 
 #include "config.h"
 
+#include <stdio.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,9 +45,11 @@
 #include <sofia-sip/sip_util.h>
 #include <sofia-sip/sip_status.h>
 
+static double parse_number(char const *str, char **return_end);
+
 /** Parse a single preference */
-int sip_prefs_parse(union sip_pref *sp, 
-		    char const **in_out_s, 
+int sip_prefs_parse(union sip_pref *sp,
+		    char const **in_out_s,
 		    int *return_negation)
 {
   char const *s;
@@ -63,7 +66,7 @@ int sip_prefs_parse(union sip_pref *sp,
     return 0;
 
   if (old_type == sp_init) {
-    if (s[0] == '\0' || 
+    if (s[0] == '\0' ||
 	strcasecmp(s, "TRUE") == 0 ||
 	strcasecmp(s, "\"TRUE\"") == 0) {
       /* Boolean */
@@ -73,7 +76,7 @@ int sip_prefs_parse(union sip_pref *sp,
       *return_negation = 0;
       *in_out_s = s + strlen(s);
       return 1;
-    } else if (strcasecmp(s, "FALSE") == 0 || 
+    } else if (strcasecmp(s, "FALSE") == 0 ||
 	       strcasecmp(s, "\"FALSE\"") == 0) {
       /* Boolean */
       sp->sp_type = sp_literal;
@@ -85,7 +88,7 @@ int sip_prefs_parse(union sip_pref *sp,
     } else if (s[0] == '"' && s[1] != '\0') {
       for (s++; IS_LWS(s[0]); s++);
     } else
-      old_type = sp_error;      
+      old_type = sp_error;
   } else if (!s[0]) {
     sp->sp_type = sp_init;
     return 0;
@@ -103,20 +106,20 @@ int sip_prefs_parse(union sip_pref *sp,
     char s0, *e;
 
     for (s++; IS_LWS(s[0]); s++);
-    
+
     s0 = s[0];
 
     if (s0 == '=')
-      sp->sp_type = sp_range, n1 = n2 = strtod(s = s + 1, &e);
+      sp->sp_type = sp_range, n1 = n2 = parse_number(s = s + 1, &e);
     else if (s0 == '<' && s[1] == '=')
-      sp->sp_type = sp_range, n1 = DBL_MIN, n2 = strtod(s = s + 2, &e);
+      sp->sp_type = sp_range, n1 = -DBL_MAX, n2 = parse_number(s = s + 2, &e);
     else if (s0 == '>' && s[1] == '=')
-      sp->sp_type = sp_range, n1 = strtod(s = s + 2, &e), n2 = DBL_MAX;
-    else if (((n1 = strtod(s, &e)) != 0.0 || s != e) && e[0] == ':')
-      sp->sp_type = sp_range, n2 = strtod(s = e + 1, &e);
+      sp->sp_type = sp_range, n1 = parse_number(s = s + 2, &e), n2 = DBL_MAX;
+    else if (((n1 = parse_number(s, &e)) != 0.0 || s != e) && e[0] == ':')
+      sp->sp_type = sp_range, n2 = parse_number(s = e + 1, &e);
     else
       /* Error in conversion */
-      sp->sp_type = sp_error, n1 = DBL_MAX, n2 = DBL_MIN;
+      sp->sp_type = sp_error, n1 = DBL_MAX, n2 = -DBL_MAX;
 
     if (s == e && (n1 == 0.0 || n2 == 0.0))
       sp->sp_type = sp_error;      /* Error in conversion */
@@ -147,16 +150,56 @@ int sip_prefs_parse(union sip_pref *sp,
   else
     old_type = sp_error;
 
-  if (old_type != sp_init && old_type != sp->sp_type) 
+  if (old_type != sp_init && old_type != sp->sp_type)
     sp->sp_type = sp_error;
-    
+
   *in_out_s = s;
 
   return sp->sp_type != sp_error;
 }
 
+/** Parse number:
+ * number          =  [ "+" / "-" ] 1*DIGIT ["." 0*DIGIT]
+ */
+static double parse_number(char const *str, char **return_end)
+{
+  double value = 0.0;
+  double decimal = 0.1;
+  char d, sign = '+';
+
+  if (return_end)
+    *return_end = (char *)str;
+
+  d = *str;
+
+  if (d == '+' || d == '-')
+    sign = d, d = *++str;
+
+  if (!('0' <= d && d <= '9'))
+    return value;
+
+  for (; '0' <= d && d <= '9'; d = *++str)
+    value = value * 10 + (d - '0');
+
+  if (d == '.') for (d = *++str; '0' <= d && d <= '9'; d = *++str) {
+    value += (d - '0') * decimal; decimal *= 0.1;
+  }
+
+  if (value > DBL_MAX)
+    value = DBL_MAX;
+
+  if (sign == '-')
+    value = -value;
+
+  if (return_end)
+    *return_end = (char *)str;
+
+  return value;
+}
+
+
 /** Return true if preferences match */
-int sip_prefs_match(union sip_pref const *a, 
+int sip_prefs_match(union sip_pref const *a,
 		    union sip_pref const *b)
 {
   if (!a || !b)
@@ -165,20 +208,20 @@ int sip_prefs_match(union sip_pref const *a,
     return 0;
   switch (a->sp_type) {
   default:
-  case sp_error: 
+  case sp_error:
     return 0;
   case sp_literal:
-    return 
+    return
       a->sp_literal.spl_length == b->sp_literal.spl_length &&
       strncasecmp(a->sp_literal.spl_value, b->sp_literal.spl_value,
 		  a->sp_literal.spl_length) == 0;
   case sp_string:
-    return 
+    return
       a->sp_string.sps_length == b->sp_string.sps_length &&
       strncmp(a->sp_string.sps_value, b->sp_string.sps_value,
 	      a->sp_string.sps_length) == 0;
   case sp_range:
-    return 
+    return
       a->sp_range.spr_lower <= b->sp_range.spr_upper &&
       a->sp_range.spr_upper >= b->sp_range.spr_lower;
   }
@@ -217,7 +260,7 @@ int sip_prefs_matching(char const *pvalue,
 
   memset(np, 0, sizeof np);
 
-  /* Usually nvalue is from Accept/Reject-Contact, 
+  /* Usually nvalue is from Accept/Reject-Contact,
      pvalue is from Contact */
   while (sip_prefs_parse(np, &nvalue, &n_negated)) {
     memset(pp, 0, sizeof pp);
@@ -248,7 +291,7 @@ int sip_prefs_matching(char const *pvalue,
   return 0;
 }
 
-/** Check if the parameter is a valid feature tag. 
+/** Check if the parameter is a valid feature tag.
  *
  * A feature tag is a parameter starting with a single plus, or a well-known
  * feature tag listed in @RFC3841: "audio", "automata", "application",
@@ -334,7 +377,7 @@ int sip_contact_is_immune(sip_contact_t const *m)
 }
 
 /**Check if @Contact matches by @AcceptContact.
- * 
+ *
  * Matching @AcceptContact and @Contact headers is done as explained in
  * @RFC3841 section 7.2.4. The caller score can be calculated from the
  * returned S and N values.
@@ -350,9 +393,9 @@ int sip_contact_is_immune(sip_contact_t const *m)
  *
  * @param m   pointer to @Contact header structure
  * @param cp   pointer to @AcceptContact header structure
- * @param return_N   return-value parameter for number of 
+ * @param return_N   return-value parameter for number of
  *                   feature tags in @AcceptContact
- * @param return_S   return-value parameter for number of 
+ * @param return_S   return-value parameter for number of
  *                   matching feature tags
  * @param return_error   return-value parameter for parsing error
  *
@@ -385,7 +428,7 @@ int sip_contact_is_immune(sip_contact_t const *m)
  * sip_contact_is_immune(), sip_contact_immunize(), sip_is_callerpref(),
  * sip_prefs_matching().
  */
-int sip_contact_accept(sip_contact_t const *m, 
+int sip_contact_accept(sip_contact_t const *m,
 		       sip_accept_contact_t const *cp,
 		       unsigned *return_S,
 		       unsigned *return_N,
@@ -416,9 +459,9 @@ int sip_contact_accept(sip_contact_t const *m,
       eq = strcspn(acc, "=");
       acc += eq + (acc[eq] == '=');
 
-      if (!sip_prefs_matching(cap, acc, return_error)) 
+      if (!sip_prefs_matching(cap, acc, return_error))
 	return 0;
-      
+
       S++;
     }
   }
@@ -426,7 +469,7 @@ int sip_contact_accept(sip_contact_t const *m,
   *return_S = S; /* Matched feature tags */
   *return_N = N; /* Number of feature tags in @AcceptContact */
 
-  return 1; 
+  return 1;
 }
 
 
@@ -441,7 +484,7 @@ int sip_contact_accept(sip_contact_t const *m,
  * @sa sip_contact_score(), sip_contact_accept(), sip_contact_immunize(),
  * sip_contact_is_immune(), @RFC3841, @RejectContact, @Contact
  */
-int sip_contact_reject(sip_contact_t const *m, 
+int sip_contact_reject(sip_contact_t const *m,
 		       sip_reject_contact_t const *reject)
 {
   unsigned S, N;
@@ -463,7 +506,7 @@ int sip_contact_reject(sip_contact_t const *m,
  *
  * @retval pointer to immunized copy if successful
  * @retval NULL upon an error
- * 
+ *
  * @sa @RFC3841, sip_is_callerpref(), sip_contact_score(),
  * sip_contact_accept(), sip_contact_reject(), @Contact
  */
@@ -498,15 +541,15 @@ sip_contact_t *sip_contact_immunize(su_home_t *home, sip_contact_t const *m)
 /** Calculate score for contact.
  *
  * The caller preference score is an integer in range of 0 to 1000.
- * 
+ *
  * @retval -1 if the contact is rejected
  * @retval 1000 if contact is immune to caller preferences
- * @retval 0..1000 reflecting @RFC3481 score in 0.000 - 1.000.
+ * @retval 0..1000 reflecting @RFC3841 score in 0.000 - 1.000.
  *
  * @sa sip_q_value(),
  * sip_contact_accept(), sip_contact_reject(), sip_contact_is_immune(),
  * sip_contact_immunize(), sip_is_callerpref(), sip_prefs_matching(),
- * @RFC3481, @AcceptContact, @RejectContact, @Contact
+ * @RFC3841, @AcceptContact, @RejectContact, @Contact
  */
 int sip_contact_score(sip_contact_t const *m,
 		      sip_accept_contact_t const *ac,
@@ -520,14 +563,14 @@ int sip_contact_score(sip_contact_t const *m,
     return 1000;		/* Immune */
 
   for (; rc; rc = rc->cp_next)
-    if (sip_contact_reject(m, rc)) 
+    if (sip_contact_reject(m, rc))
       break;
   if (rc)
     return -1;			/* Rejected */
 
   for (; ac; ac = ac->cp_next) {
     unsigned S, N;
-	
+
     if (!sip_contact_accept(m, ac, &S, &N, &error)) {
       if (ac->cp_require)
 	return 0;		/* Discarded */
@@ -543,11 +586,11 @@ int sip_contact_score(sip_contact_t const *m,
 	return 0;		/* Dropped */
     }
 
-    if (S > 0 && N > 0) 
+    if (S > 0 && N > 0)
       S_total += sip_q_value(ac->cp_q) * (scale * S / N + (2 * S >= N));
   }
 
-  if (!M) 
+  if (!M)
     return 0;
 
   S_total /= M;
