@@ -31,7 +31,7 @@
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>
  * @author Martti Mela <Martti.Mela@nokia.com>
- * 
+ *
  * @date Created: Tue Sep 14 15:51:04 1999 ppessi
  */
 
@@ -213,11 +213,11 @@ typedef struct su_root_s su_root_t;
 /**Default type of application context for <a href="#su_root_t">su_root_t</a>.
  *
  * Application may define the typedef ::su_root_magic_t to appropriate type
- * by defining macro SU_ROOT_MAGIC_T before including <su_wait.h>, for
+ * by defining macro SU_ROOT_MAGIC_T before including <sofia-sip/su_wait.h>, for
  * example,
  * @code
  * #define SU_ROOT_MAGIC_T struct context
- * #include <su_wait.h>
+f * #include <sofia-sip/su_wait.h>
  * @endcode
  */
 #define SU_ROOT_MAGIC_T void
@@ -226,41 +226,41 @@ typedef struct su_root_s su_root_t;
 /** <a href="#su_root_t">Root context</a> pointer type.
  *
  * Application may define the typedef ::su_root_magic_t to appropriate type
- * by defining macro SU_ROOT_MAGIC_T () before including <su_wait.h>, for
+ * by defining macro SU_ROOT_MAGIC_T () before including <sofia-sip/su_wait.h>, for
  * example,
  * @code
  * #define SU_ROOT_MAGIC_T struct context
- * #include <su_wait.h>
+ * #include <sofia-sip/su_wait.h>
  * @endcode
  */
 typedef SU_ROOT_MAGIC_T su_root_magic_t;
 
 #ifndef SU_WAKEUP_ARG_T
-/**Default type of @link ::su_wakeup_f wakeup function @endlink 
- * @link ::su_wakeup_arg_t argument type @endlink.  
+/**Default type of @link ::su_wakeup_f wakeup function @endlink
+ * @link ::su_wakeup_arg_t argument type @endlink.
  *
  * The application can define the typedef ::su_wakeup_arg_t by defining
- * the SU_WAKEUP_ARG_T () before including <su_wait.h>, for example,
+ * the SU_WAKEUP_ARG_T () before including <sofia-sip/su_wait.h>, for example,
  * @code
  * #define SU_WAKEUP_ARG_T struct transport
- * #include <su_wait.h>
+ * #include <sofia-sip/su_wait.h>
  * @endcode
  */
 #define SU_WAKEUP_ARG_T void
 #endif
 
-/** @link ::su_wakeup_f Wakeup callback @endlink argument type. 
+/** @link ::su_wakeup_f Wakeup callback @endlink argument type.
  *
  * The application can define the typedef ::su_wakeup_arg_t by defining
- * the SU_WAKEUP_ARG_T () before including <su_wait.h>, for example,
+ * the SU_WAKEUP_ARG_T () before including <sofia-sip/su_wait.h>, for example,
  * @code
  * #define SU_WAKEUP_ARG_T struct transport
- * #include <su_wait.h>
+ * #include <sofia-sip/su_wait.h>
  * @endcode
  */
 typedef SU_WAKEUP_ARG_T su_wakeup_arg_t;
 
-/** Wakeup callback function pointer type. 
+/** Wakeup callback function pointer type.
  *
  * Whenever a registered wait object receives an event, the @link
  * ::su_wakeup_f callback function @endlink is invoked.
@@ -269,7 +269,7 @@ typedef int (*su_wakeup_f)(su_root_magic_t *,
 			   su_wait_t *,
 			   su_wakeup_arg_t *arg);
 
-enum { 
+enum {
   su_pri_normal,		/**< Normal priority */
   su_pri_first,			/**< Elevated priority */
   su_pri_realtime		/**< Real-time priority */
@@ -287,11 +287,11 @@ SOFIAPUBVAR int su_root_size_hint;
 /**Default type of application context for prepoll function.
  *
  * Application may define the typedef ::su_prepoll_magic_t to appropriate type
- * by defining macro #SU_PREPOLL_MAGIC_T before including <su_wait.h>, for
+ * by defining macro #SU_PREPOLL_MAGIC_T before including <sofia-sip/su_wait.h>, for
  * example,
  * @code
  * #define SU_PREPOLL_MAGIC_T struct context
- * #include <su_wait.h>
+ * #include <sofia-sip/su_wait.h>
  * @endcode
  */
 #define SU_PREPOLL_MAGIC_T void
@@ -300,19 +300,19 @@ SOFIAPUBVAR int su_root_size_hint;
 /** <a href="#su_root_t">Root context</a> pointer type.
  *
  * Application may define the typedef ::su_prepoll_magic_t to appropriate type
- * by defining macro #SU_PREPOLL_MAGIC_T before including <su_wait.h>, for
+ * by defining macro #SU_PREPOLL_MAGIC_T before including <sofia-sip/su_wait.h>, for
  * example,
  * @code
  * #define SU_PREPOLL_MAGIC_T struct context
- * #include <su_wait.h>
+ * #include <sofia-sip/su_wait.h>
  * @endcode
  */
 typedef SU_PREPOLL_MAGIC_T su_prepoll_magic_t;
 
 
-/** Pre-poll callback function prototype. 
+/** Pre-poll callback function prototype.
+ *
  *
- * 
  */
 typedef void su_prepoll_f(su_prepoll_magic_t *, su_root_t *);
 
@@ -326,8 +326,8 @@ typedef void su_prepoll_f(su_prepoll_magic_t *, su_root_t *);
 #ifndef SU_TIMER_ARG_T
 /** Default type of timer expiration callback function argument type.
  * Application may define this to appropriate type before including
- * <su_wait.h>. */
-#define SU_TIMER_ARG_T void 
+ * <sofia-sip/su_wait.h>. */
+#define SU_TIMER_ARG_T void
 #endif
 
 /** Timer object type. */
@@ -337,10 +337,16 @@ typedef struct su_timer_s su_timer_t;
 typedef SU_TIMER_ARG_T su_timer_arg_t;
 
 /** Timeout function type. */
-typedef void (*su_timer_f)(su_root_magic_t *magic, 
+typedef void (*su_timer_f)(su_root_magic_t *magic,
 			   su_timer_t *t,
 			   su_timer_arg_t *arg);
 
+#ifndef SU_TIMER_QUEUE_T
+#define SU_TIMER_QUEUE_T su_timer_t *
+#endif
+
+typedef SU_TIMER_QUEUE_T su_timer_queue_t;
+
 /* ---------------------------------------------------------------------- */
 
 /* Tasks */
@@ -364,9 +370,9 @@ typedef _su_task_t const *_su_task_r;
 /* Messages */
 #ifndef SU_MSG_ARG_T
 /** Default type of su_msg_t message data.  Application may define this to
- * appropriate type before including <su_wait.h>.
+ * appropriate type before including <sofia-sip/su_wait.h>.
  */
-#define SU_MSG_ARG_T void 
+#define SU_MSG_ARG_T void
 #endif
 
 /** Message argument type. */
@@ -378,17 +384,23 @@ typedef struct su_msg_s su_msg_t;
 /** Message reference type. */
 typedef su_msg_t *su_msg_r[1];
 
-/** Contstant reference to su_msg */
+/** Constant reference to su_msg.
+ *
+ * @note The reference is constant, but the message can be modified.
+ */
 typedef su_msg_t * const su_msg_cr[1];
 
 /** Initializer for a message reference. @HI */
 #define SU_MSG_R_INIT   { NULL }
 
 /** Message delivery function type. */
-typedef void su_msg_function(su_root_magic_t *magic, 
+typedef void su_msg_function(su_root_magic_t *magic,
 			     su_msg_r msg,
 			     su_msg_arg_t *arg);
 
+/** Message deinitializer function type. @NEW_1_12_8 */
+typedef void su_msg_deinit_function(su_msg_arg_t *arg);
+
 /** Message delivery function pointer type. */
 typedef su_msg_function *su_msg_f;
 
@@ -442,14 +454,14 @@ SOFIAPUBFUN void su_root_destroy(su_root_t*);
 SOFIAPUBFUN char const *su_root_name(su_root_t *self);
 SOFIAPUBFUN int su_root_set_magic(su_root_t *self, su_root_magic_t *magic);
 SOFIAPUBFUN su_root_magic_t *su_root_magic(su_root_t *root);
-SOFIAPUBFUN int su_root_register(su_root_t*, su_wait_t *, 
+SOFIAPUBFUN int su_root_register(su_root_t*, su_wait_t *,
 				 su_wakeup_f, su_wakeup_arg_t *,
 				 int priority);
 /* This is slow. Deprecated. */
-SOFIAPUBFUN int su_root_unregister(su_root_t*, su_wait_t *, 
+SOFIAPUBFUN int su_root_unregister(su_root_t*, su_wait_t *,
 				   su_wakeup_f, su_wakeup_arg_t*);
 SOFIAPUBFUN int su_root_deregister(su_root_t*, int);
-SOFIAPUBFUN int su_root_eventmask(su_root_t *, 
+SOFIAPUBFUN int su_root_eventmask(su_root_t *,
 				  int index, int socket, int events);
 SOFIAPUBFUN su_duration_t su_root_step(su_root_t *root, su_duration_t timeout);
 SOFIAPUBFUN su_duration_t su_root_sleep(su_root_t *root, su_duration_t);
@@ -459,8 +471,8 @@ SOFIAPUBFUN void su_root_break(su_root_t *root);
 SOFIAPUBFUN _su_task_r su_root_task(su_root_t const *root);
 SOFIAPUBFUN _su_task_r su_root_parent(su_root_t const *root);
 
-SOFIAPUBFUN int su_root_add_prepoll(su_root_t *root, 
-				    su_prepoll_f *, 
+SOFIAPUBFUN int su_root_add_prepoll(su_root_t *root,
+				    su_prepoll_f *,
 				    su_prepoll_magic_t *);
 SOFIAPUBFUN int su_root_remove_prepoll(su_root_t *root);
 
@@ -468,6 +480,10 @@ SOFIAPUBFUN struct _GSource *su_root_gsource(su_root_t *self);
 
 SOFIAPUBFUN int su_root_yield(su_root_t *root);
 
+SOFIAPUBFUN int su_root_release(su_root_t *root);
+SOFIAPUBFUN int su_root_obtain(su_root_t *root);
+SOFIAPUBFUN int su_root_has_thread(su_root_t *root);
+
 /* Timers */
 SOFIAPUBFUN su_timer_t *su_timer_create(su_task_r const, su_duration_t msec)
      __attribute__((__malloc__));
@@ -478,13 +494,13 @@ SOFIAPUBFUN int su_timer_set_interval(su_timer_t *t, su_timer_f,
 SOFIAPUBFUN int su_timer_set_at(su_timer_t *, su_timer_f,
 				su_timer_arg_t *, su_time_t);
 SOFIAPUBFUN int su_timer_run(su_timer_t *, su_timer_f, su_timer_arg_t *);
-SOFIAPUBFUN int su_timer_set_for_ever(su_timer_t *, su_timer_f, 
+SOFIAPUBFUN int su_timer_set_for_ever(su_timer_t *, su_timer_f,
 				      su_timer_arg_t *);
 SOFIAPUBFUN int su_timer_reset(su_timer_t *);
 
 SOFIAPUBFUN su_root_t *su_timer_root(su_timer_t const *);
 
-SOFIAPUBFUN int su_timer_expire(su_timer_t ** const, 
+SOFIAPUBFUN int su_timer_expire(su_timer_queue_t * const,
 				su_duration_t *tout,
 				su_time_t now);
 
@@ -502,7 +518,7 @@ SOFIAPUBFUN int su_task_cmp(su_task_r const, su_task_r const);
 SOFIAPUBFUN int su_task_is_running(su_task_r const);
 
 SOFIAPUBFUN su_root_t *su_task_root(su_task_r const self);
-SOFIAPUBFUN su_timer_t **su_task_timers(su_task_r const self);
+SOFIAPUBFUN su_timer_queue_t *su_task_timers(su_task_r const self);
 
 SOFIAPUBFUN int su_task_execute(su_task_r const task,
 				int (*function)(void *), void *arg,
@@ -510,10 +526,11 @@ SOFIAPUBFUN int su_task_execute(su_task_r const task,
 
 /* Messages */
 SOFIAPUBFUN int su_msg_create(su_msg_r msg,
-			      su_task_r const to, su_task_r const from, 
+			      su_task_r const to, su_task_r const from,
 			      su_msg_f wakeup, isize_t size);
 SOFIAPUBFUN int su_msg_report(su_msg_r msg, su_msg_f report);
-SOFIAPUBFUN int su_msg_reply(su_msg_r reply, su_msg_r const msg,
+SOFIAPUBFUN int su_msg_deinitializer(su_msg_r msg, su_msg_deinit_function *);
+SOFIAPUBFUN int su_msg_reply(su_msg_r reply, su_msg_cr msg,
 			     su_msg_f wakeup, isize_t size);
 SOFIAPUBFUN void su_msg_destroy(su_msg_r msg);
 SOFIAPUBFUN void su_msg_save(su_msg_r msg, su_msg_r msg0);
@@ -524,6 +541,11 @@ SOFIAPUBFUN _su_task_r su_msg_from(su_msg_cr msg);
 SOFIAPUBFUN _su_task_r su_msg_to(su_msg_cr msg);
 SOFIAPUBFUN int su_msg_send(su_msg_r msg);
 
+SOFIAPUBFUN int su_msg_new(su_msg_r msg, size_t size);
+SOFIAPUBFUN int su_msg_send_to(su_msg_r msg,
+			       su_task_r const to,
+			       su_msg_f wakeup);
+
 /** Does reference contain a message? */
 #if SU_HAVE_INLINE
 static SU_INLINE
@@ -537,10 +559,10 @@ int su_msg_is_non_null(su_msg_cr msg)
 
 /* Clones */
 SOFIAPUBFUN int su_root_threading(su_root_t *self, int enable);
-SOFIAPUBFUN int su_clone_start(su_root_t *root, 
+SOFIAPUBFUN int su_clone_start(su_root_t *root,
 			       su_clone_r,
 			       su_root_magic_t *magic,
-			       su_root_init_f, 
+			       su_root_init_f,
 			       su_root_deinit_f);
 SOFIAPUBFUN _su_task_r su_clone_task(su_clone_r);
 SOFIAPUBFUN void su_clone_forget(su_clone_r);
