@@ -23,12 +23,12 @@
  */
 
 /**@ingroup su_wait
- * @CFILE su_root.c 
- * OS-independent synchronization interface. 
+ * @CFILE su_root.c
+ * OS-independent synchronization interface.
  * @internal
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>
- * 
+ *
  * @date Created: Tue Sep 14 15:51:04 1999 ppessi
  */
 
@@ -56,7 +56,7 @@ struct su_root_s;
 #include "sofia-sip/su_alloc.h"
 
 /**@ingroup su_wait
- * 
+ *
  * @page su_root_t Tasks and root objects
  *
  * A task is the basic execution unit for the Sofia event-driven programming
@@ -91,8 +91,8 @@ struct su_root_s;
  * New tasks can be created via su_clone_start() function.
  */
 
-/**@ingroup su_wait 
- * 
+/**@ingroup su_wait
+ *
  * @page su_root_register Registering Wait Objects
  *
  * When application expects I/O events, it can create a wait object and
@@ -111,8 +111,8 @@ struct su_root_s;
  */
 
 /**@ingroup su_wait
- * 
- * Contains hint of number of sockets supported by su_root_t */ 
+ *
+ * Contains hint of number of sockets supported by su_root_t */
 int su_root_size_hint = 64;
 
 /* =========================================================================
@@ -187,7 +187,7 @@ _su_task_r su_task_new(su_task_r task, su_root_t *root, su_port_t *port)
 void su_task_copy(su_task_r dst, su_task_r const src)
 {
   su_port_t *port;
-  
+
   assert(src); assert(dst);
 
   SU_TASK_ZAP(dst, su_task_copy);
@@ -219,10 +219,10 @@ void su_task_move(su_task_r dst, su_task_r src)
 
 /**
  * Compare two tasks with each other.
- * 
+ *
  * @param a  First task
  * @param b  Second task
- * 
+ *
  * @retval negative number, if a < b
  * @retval positive number, if a > b
  * @retval 0, if a == b.
@@ -249,14 +249,14 @@ int su_task_cmp(su_task_r const a, su_task_r const b)
  *
  * @param task  task handle
  *
- * @retval true (nonzero) if task is not stopped, 
+ * @retval true (nonzero) if task is not stopped,
  * @retval zero if it is null or stopped.
  */
 int su_task_is_running(su_task_r const task)
 {
-  return 
-    task && 
-    task->sut_port && 
+  return
+    task &&
+    task->sut_port &&
     task->sut_root;
 }
 
@@ -265,7 +265,7 @@ int su_task_is_running(su_task_r const task)
  *
  * @param self task handle
  * @param root pointer to the root object
- * 
+ *
  * @retval 0 if successful,
  * @retval -1 otherwise.
  */
@@ -275,16 +275,16 @@ int su_task_attach(su_task_r self, su_root_t *root)
     self->sut_root = root;
     return 0;
   }
-  else 
+  else
     return -1;
 }
 
-/** 
+/**
  * Get root pointer attached to a task handle.
  *
  * @param self task handle
  *
- * @return 
+ * @return
  * A pointer to root object attached to the task handle, or NULL if no root
  * object has been attached.
  */
@@ -307,7 +307,7 @@ int su_task_detach(su_task_r self)
 
 /**
  * Return the timer list associated with given task.
- * 
+ *
  * @param task task handle
  *
  * @return A timer list of the task. If there are no timers, it returns
@@ -327,9 +327,14 @@ int su_task_execute(su_task_r const task,
 		    int (*function)(void *), void *arg,
 		    int *return_value)
 {
+  int dummy;
+
   if (function == NULL)
     return (errno = EFAULT), -1;
 
+  if (return_value == NULL)
+    return_value = &dummy;
+
   if (!su_port_own_thread(task->sut_port)) {
     return su_port_execute(task, function, arg, return_value);
   }
@@ -403,7 +408,7 @@ su_root_t *su_root_create_with_port(su_root_magic_t *magic,
     /* This one creates a new reference to port */
     su_task_new(self->sur_task, self, port);
     /* ... so we zap the old one below */
-  } 
+  }
 
   su_port_decref(port, "su_root_create_with_port");
 
@@ -411,7 +416,7 @@ su_root_t *su_root_create_with_port(su_root_magic_t *magic,
 }
 
 /** Destroy a root object.
- * 
+ *
  *  Stop and free an instance of su_root_t
  *
  * @param self     pointer to a root object.
@@ -536,7 +541,7 @@ struct _GSource *su_root_gsource(su_root_t *self)
   return su_port_gsource(self->sur_port);
 }
 
-/** Register a su_wait_t object. 
+/** Register a su_wait_t object.
  *
  *  The function su_root_register() registers a su_wait_t object. The wait
  *  object, a callback function and a argument are stored to the root
@@ -650,7 +655,7 @@ int su_root_eventmask(su_root_t *self, int index, int socket, int events)
  *
  * @param self      pointer to root object
  * @param multishot multishot mode (0 => disables, 1 => enables, -1 => query)
- * 
+ *
  * @retval 0 multishot mode is disabled
  * @retval 1 multishot mode is enabled
  * @retval -1 an error occurred
@@ -803,8 +808,8 @@ _su_task_r su_root_parent(su_root_t const *self)
 }
 
 /** Add a pre-poll callback. */
-int su_root_add_prepoll(su_root_t *root, 
-			su_prepoll_f *callback, 
+int su_root_add_prepoll(su_root_t *root,
+			su_prepoll_f *callback,
 			su_prepoll_magic_t *magic)
 {
   if (root == NULL)
@@ -835,7 +840,7 @@ int su_root_release(su_root_t *root)
   return su_port_release(root->sur_port);
 }
 
-/** Obtain the root port from other thread. 
+/** Obtain the root port from other thread.
  *
  * @param root pointer to root object
  *
@@ -843,7 +848,7 @@ int su_root_release(su_root_t *root)
  * @retval -1 upon an error
  *
  * @ERRORS
- * @ERROR EFAULT 
+ * @ERROR EFAULT
  * @NEW_1_12_7
  */
 int su_root_obtain(su_root_t *root)
@@ -853,7 +858,7 @@ int su_root_obtain(su_root_t *root)
   return su_port_obtain(root->sur_port);
 }
 
-/**Check if a thread has obtained the root. 
+/**Check if a thread has obtained the root.
  *
  * @param root a pointer to root object
  *
@@ -875,6 +880,32 @@ int su_root_has_thread(su_root_t *root)
  * Messages
  */
 
+/**
+ * Allocate a su message of given size.
+ *
+ * Allocate a su message with given data size.
+ *
+ * @param  rmsg   handle to the new message (may be uninitialized prior calling)
+ * @param  size   size of the message data
+ *
+ * @retval  0 if successful,
+ * @retval -1 if message allocation fails.
+ *
+ * @NEW_1_12_8
+ */
+int su_msg_new(su_msg_r rmsg, size_t size)
+{
+  su_msg_t *msg;
+  size_t total = sizeof(*msg) + (size_t)size;
+
+  *rmsg = msg = su_zalloc(NULL, (isize_t)total);
+  if (!*rmsg)
+    return -1;
+
+  msg->sum_size = total;
+  return 0;
+}
+
 /**
  * Allocates a message of given size.
  *
@@ -888,7 +919,7 @@ int su_root_has_thread(su_root_t *root)
  * @param  size   size of the message data
  *
  * @retval  0 if successful,
- * @retval -1 if message allocation fails.  
+ * @retval -1 if message allocation fails.
  */
 int su_msg_create(su_msg_r        rmsg,
 		  su_task_r const to,
@@ -896,29 +927,22 @@ int su_msg_create(su_msg_r        rmsg,
 		  su_msg_f        wakeup,
 		  isize_t         size)
 {
-  su_msg_t *msg;
-
-  msg = su_zalloc(NULL, sizeof(*msg) + size);
-
-  if (msg) {
-    msg->sum_size = sizeof(*msg) + size;
-    SU_TASK_COPY(msg->sum_to, to, su_msg_create);
-    SU_TASK_COPY(msg->sum_from, from, su_msg_create);
-    msg->sum_func = wakeup;
-    *rmsg = msg;
+  if (su_msg_new(rmsg, (size_t) size) == 0) {
+    SU_TASK_COPY(rmsg[0]->sum_to, to, su_msg_create);
+    SU_TASK_COPY(rmsg[0]->sum_from, from, su_msg_create);
+    rmsg[0]->sum_func = wakeup;
     return 0;
-  } 
+  }
 
-  *rmsg = NULL;
   return -1;
 }
 
 /** Add a delivery report function to a message.
- * 
+ *
  * The delivery report funcgtion gets called by the sending task after the
  * message was delivered and the message function was executed. (The
- * su_root_t message delivery loop calls su_msg_delivery_report() 
- * 
+ * su_root_t message delivery loop calls su_msg_delivery_report()
+ *
  */
 int su_msg_report(su_msg_r msg,
 		  su_msg_f report)
@@ -931,11 +955,33 @@ int su_msg_report(su_msg_r msg,
   return -1;
 }
 
+/** Add a deinitializer function to a message.
+ *
+ * The deinitializer function is called when the message gets destroyed. It
+ * is called even if the message was never delivered. Note that the thread
+ * destroying the message and calling the deinit function is not necessarily
+ * the same that sent the message nor the original recipient.
+ *
+ * @param rmsg   message reference
+ * @param deinit pointer to deinitializer function
+ *
+ * @NEW_1_12_8
+ */
+int su_msg_deinitializer(su_msg_r rmsg,
+			 su_msg_deinit_function *deinit)
+{
+  if (rmsg && rmsg[0]) {
+    rmsg[0]->sum_deinit = deinit;
+    return 0;
+  }
+  return -1;
+}
+
 /**
  * Allocates a reply message of given size.
  *
  * @param reply     handle to the new message (may be uninitialized prior calling)
- * @param msg       the incoming message
+ * @param rmsg       the incoming message
  * @param wakeup    function that is called when message is delivered
  * @param size      size of the message data
  *
@@ -943,17 +989,17 @@ int su_msg_report(su_msg_r msg,
  * @retval -1 otherwise.
  */
 
-int su_msg_reply(su_msg_r reply, su_msg_cr msg,
+int su_msg_reply(su_msg_r reply, su_msg_cr rmsg,
 		 su_msg_f wakeup, isize_t size)
 {
-  su_msg_r msg0;
+  su_msg_r rmsg0;
 
-  assert(msg != reply);
+  assert(rmsg != reply);
 
-  *msg0 = *(su_msg_t **) msg;
+  *rmsg0 = *(su_msg_t **) rmsg;
   *reply = NULL;
 
-  return su_msg_create(reply, su_msg_from(msg0), su_msg_to(msg0), wakeup, size);
+  return su_msg_create(reply, su_msg_from(rmsg0), su_msg_to(rmsg0), wakeup, size);
 }
 
 
@@ -964,38 +1010,38 @@ int su_msg_reply(su_msg_r reply, su_msg_cr msg,
  * sending task. The sending task calls the delivery report function when it
  * has received the message.
  */
-void su_msg_delivery_report(su_msg_r msg)
+void su_msg_delivery_report(su_msg_r rmsg)
 {
   su_task_r swap;
 
-  if (!msg || !msg[0])
+  if (!rmsg || !rmsg[0])
     return;
 
-  if (!msg[0]->sum_report) {
-    su_msg_destroy(msg);
+  if (!rmsg[0]->sum_report) {
+    su_msg_destroy(rmsg);
     return;
   }
 
-  *swap = *msg[0]->sum_from;
-  *msg[0]->sum_from = *msg[0]->sum_to;
-  *msg[0]->sum_to = *swap;
+  *swap = *rmsg[0]->sum_from;
+  *rmsg[0]->sum_from = *rmsg[0]->sum_to;
+  *rmsg[0]->sum_to = *swap;
 
-  msg[0]->sum_func = msg[0]->sum_report;
-  msg[0]->sum_report = NULL;
-  su_msg_send(msg);
+  rmsg[0]->sum_func = rmsg[0]->sum_report;
+  rmsg[0]->sum_report = NULL;
+  su_msg_send(rmsg);
 }
 
 /** Save a message. */
-void su_msg_save(su_msg_r save, su_msg_r msg)
+void su_msg_save(su_msg_r save, su_msg_r rmsg)
 {
   if (save) {
-    if (msg)
-      save[0] = msg[0];
+    if (rmsg)
+      save[0] = rmsg[0];
     else
       save[0] = NULL;
   }
-  if (msg)
-    msg[0] = NULL;
+  if (rmsg)
+    rmsg[0] = NULL;
 }
 
 /**
@@ -1005,19 +1051,24 @@ void su_msg_save(su_msg_r save, su_msg_r msg)
  */
 void su_msg_destroy(su_msg_r rmsg)
 {
+  su_msg_t *msg;
+
   assert(rmsg);
 
-  if (rmsg[0]) {
-    SU_TASK_ZAP(rmsg[0]->sum_to, su_msg_destroy);
-    SU_TASK_ZAP(rmsg[0]->sum_from, su_msg_destroy);
+  msg = rmsg[0], rmsg[0] = NULL;
 
-    su_free(NULL, rmsg[0]);
+  if (msg) {
+    SU_TASK_ZAP(msg->sum_to, su_msg_destroy);
+    SU_TASK_ZAP(msg->sum_from, su_msg_destroy);
+
+    if (msg->sum_deinit)
+      msg->sum_deinit(msg->sum_data);
+
+    su_free(NULL, msg);
   }
-
-  rmsg[0] = NULL;
 }
 
-/** Gets a pointer to the message data area. 
+/** Gets a pointer to the message data area.
  *
  * The function @c su_msg_data() returns a pointer to the message data
  * area. If @c rmsg contains a @c NULL handle, or message size is 0, @c NULL
@@ -1025,7 +1076,7 @@ void su_msg_destroy(su_msg_r rmsg)
  *
  * @param rmsg       message handle
  *
- * @return A pointer to the message data area is returned.  
+ * @return A pointer to the message data area is returned.
  */
 su_msg_arg_t *su_msg_data(su_msg_cr rmsg)
 {
@@ -1048,13 +1099,13 @@ isize_t su_msg_size(su_msg_cr rmsg)
  * If the message handle contains NULL the function @c su_msg_from
  * returns NULL.
  *
- * @param msg       message handle
+ * @param rmsg       message handle
  *
- * @return The task handle of the sender is returned.  
+ * @return The task handle of the sender is returned.
  */
-_su_task_r su_msg_from(su_msg_cr msg)
+_su_task_r su_msg_from(su_msg_cr rmsg)
 {
-  return msg[0] ? msg[0]->sum_from : NULL;
+  return rmsg[0] ? rmsg[0]->sum_from : NULL;
 }
 
 /** Get destination task.
@@ -1065,28 +1116,28 @@ _su_task_r su_msg_from(su_msg_cr msg)
  * If the message handle contains NULL the function @c su_msg_to
  * returns NULL.
  *
- * @param msg       message handle
+ * @param rmsg       message handle
  *
- * @return The task handle of the recipient is returned.  
+ * @return The task handle of the recipient is returned.
  */
-_su_task_r su_msg_to(su_msg_cr msg)
+_su_task_r su_msg_to(su_msg_cr rmsg)
 {
-  return msg[0] ? msg[0]->sum_to : NULL;
+  return rmsg[0] ? rmsg[0]->sum_to : NULL;
 }
 
-/** Remove references to 'from' and 'to' tasks from a message. 
+/** Remove references to 'from' and 'to' tasks from a message.
  *
- * @param msg       message handle
+ * @param rmsg       message handle
  */
-void su_msg_remove_refs(su_msg_cr msg)
+void su_msg_remove_refs(su_msg_cr rmsg)
 {
-  if (msg[0]) {
-    su_task_deinit(msg[0]->sum_to);
-    su_task_deinit(msg[0]->sum_from);
+  if (rmsg[0]) {
+    su_task_deinit(rmsg[0]->sum_to);
+    su_task_deinit(rmsg[0]->sum_from);
   }
 }
 
-/**Send a message. 
+/**Send a message.
  *
  * The function @c su_msg_send() sends the message. The message is added to
  * the recipients message queue, and recipient is waken up. The caller may
@@ -1105,7 +1156,7 @@ int su_msg_send(su_msg_r rmsg)
   if (rmsg[0]) {
     su_msg_t *msg = rmsg[0];
 
-    if (msg->sum_to->sut_port) 
+    if (msg->sum_to->sut_port)
       return su_port_send(msg->sum_to->sut_port, rmsg);
 
     su_msg_destroy(rmsg);
@@ -1113,5 +1164,41 @@ int su_msg_send(su_msg_r rmsg)
     return -1;
   }
 
-  return 0;		
+  return 0;
+}
+
+/** Send message to the @a to_task and mark @a from_task as sender.
+ *
+ * @NEW_1_12_8
+ */
+SOFIAPUBFUN int su_msg_send_to(su_msg_r rmsg,
+			       su_task_r const to_task,
+			       su_msg_f wakeup)
+{
+  assert(rmsg); assert(to_task);
+
+  if (rmsg[0]) {
+    su_msg_t *msg = rmsg[0];
+
+    if (wakeup)
+      msg->sum_func = wakeup;
+
+    if (msg->sum_to->sut_port &&
+	msg->sum_to->sut_port != to_task->sut_port) {
+      SU_TASK_ZAP(msg->sum_to, "su_msg_send_to");
+    }
+
+    if (to_task->sut_port != NULL) {
+      msg->sum_to->sut_port = NULL;
+      msg->sum_to->sut_root = to_task->sut_root;
+
+      return su_port_send(to_task->sut_port, rmsg);
+    }
+
+    su_msg_destroy(rmsg);
+    errno = EINVAL;
+    return -1;
+  }
+
+  return 0;
 }
