@@ -22,7 +22,7 @@
  *
  */
 
-/**
+/**@internal
  * @file torture_su_port.c
  * @brief Test su_poll_port interface
  *
@@ -59,7 +59,11 @@ int tstflags;
 
 char const *name = "torture_su_port";
 
+#if HAVE_OPEN_C
+int const N0 = SU_MBOX_SIZE > 0, N = 63, I = 64;
+#else
 int const N0 = SU_MBOX_SIZE > 0, N = 128, I = 129;
+#endif
 
 int test_sup_indices(su_port_t const *port)
 {
@@ -81,7 +85,7 @@ int test_sup_indices(su_port_t const *port)
 
   if (n != port->sup_n_waits)
     return 0;
-  
+
   n = 0;
 
   for (i = 1; i <= N; i++) {
@@ -104,19 +108,19 @@ int test_sup_indices(su_port_t const *port)
   return n == port->sup_size_waits;
 }
 
-struct su_root_magic_s 
+struct su_root_magic_s
 {
   int error, *sockets, *regs, *wakeups;
 };
 
-static int callback(su_root_magic_t *magic, 
+static int callback(su_root_magic_t *magic,
 		    su_wait_t *w,
 		    su_wakeup_arg_t *arg)
 {
   intptr_t i = (intptr_t)arg;
 
   assert(magic);
- 
+
   if (i <= 0 || i > I)
     return ++magic->error;
 
@@ -193,7 +197,7 @@ int test_register(void)
   su_root_size_hint = 16;
 
   TEST_1(port = su_poll_port_create());
-  TEST(su_port_threadsafe(port), 0);
+  TEST(su_home_threadsafe(su_port_home(port)), 0);
   /* Before 1.12.4 su_port_create() had reference count 0 after creation */
   /* su_port_incref(port, "test_register"); */
 
@@ -204,7 +208,7 @@ int test_register(void)
 
     if (bind(sockets[i], &su->su_sa, sizeof su->su_sin) != 0)
       perror("bind"), assert(0);
-    
+
     TEST(su_wait_create(wait + i, sockets[i], SU_WAIT_IN), 0);
 
     reg[i] = su_port_register(port, root, wait + i, callback, (void*)i, 0);
@@ -281,7 +285,7 @@ int test_register(void)
 
 #if HAVE_EPOLL
   /* With epoll we do not bother to prioritize the wait list */
-  if (port->sup_epoll == -1) 
+  if (port->sup_epoll == -1)
 #endif
     TEST(port->sup_pri_offset, prioritized);
 
@@ -331,6 +335,10 @@ int main(int argc, char *argv[])
       usage(1);
   }
 
+#if HAVE_OPEN_C
+  tstflags |= tst_verbatim;
+#endif
+
   su_init();
 
   retval |= test_register(); fflush(stdout);
