@@ -35,6 +35,7 @@
 #include "config.h"
 
 #include "tport_internal.h"
+#include "sofia-sip/hostdomain.h"
 
 #if HAVE_IP_RECVERR || HAVE_IPV6_RECVERR
 #include <linux/types.h>
@@ -43,6 +44,7 @@
 #endif
 
 #include <stdlib.h>
+#include <string.h>
 #include <time.h>
 #include <assert.h>
 #include <errno.h>
@@ -58,6 +60,13 @@ int tport_udp_init_client(tport_primary_t *pri,
 			  tagi_t const *tags,
 			  char const **return_culprit);
 
+#if HAVE_FUNC
+#elif HAVE_FUNCTION
+#define __func__ __FUNCTION__
+#else
+static char const __func__[] = "tport_type_udp";
+#endif
+
 tport_vtable_t const tport_udp_client_vtable =
 {
   "udp", tport_type_client,
@@ -105,6 +114,10 @@ int tport_udp_init_primary(tport_primary_t *pri,
   unsigned rmem = 0, wmem = 0;
   int events = SU_WAIT_IN;
   int s;
+#if HAVE_IP_ADD_MEMBERSHIP
+  su_sockaddr_t *su = (su_sockaddr_t *)ai->ai_addr;
+#endif
+  int const one = 1; (void)one;
 
   s = su_socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
   if (s == INVALID_SOCKET)
@@ -115,39 +128,89 @@ int tport_udp_init_primary(tport_primary_t *pri,
   if (tport_bind_socket(s, ai, return_culprit) < 0)
     return -1;
 
+  tport_set_tos(s, ai, pri->pri_params->tpp_tos);
+
+#if HAVE_IP_ADD_MEMBERSHIP
+  if (ai->ai_family == AF_INET &&
+      IN_MULTICAST(ntohl(su->su_sin.sin_addr.s_addr))) {
+    /* Try to join to the multicast group */
+    /* Bind to the SIP address like
+       <sip:88.77.66.55:5060;maddr=224.0.1.75;transport=udp> */
+    struct ip_mreq imr[1];
+    struct in_addr iface;
+
+    memset(imr, 0, sizeof imr);
+
+    imr->imr_multiaddr = su->su_sin.sin_addr;
+
+    if (host_is_ip4_address(tpn->tpn_canon) &&
+	su_inet_pton(AF_INET, tpn->tpn_canon, &iface) > 0) {
+      imr->imr_interface = iface;
+    }
+
+    if (setsockopt(s, IPPROTO_IP, IP_ADD_MEMBERSHIP, imr, (sizeof imr)) < 0) {
+      SU_DEBUG_3(("setsockopt(%s): %s\n",
+		  "IP_ADD_MEMBERSHIP", su_strerror(su_errno())));
+    }
+#if HAVE_IP_MULTICAST_LOOP
+    else
+      if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_LOOP, &one, sizeof one) < 0) {
+	SU_DEBUG_3(("setsockopt(%s): %s\n",
+		    "IP_MULTICAST_LOOP", su_strerror(su_errno())));
+    }
+#endif
+  }
+#endif
+
+#if HAVE_IP_MTU_DISCOVER
+  {
+    /* Turn off DF flag on Linux */
+    int dont = IP_PMTUDISC_DONT;
+    if (setsockopt(s, IPPROTO_IP, IP_MTU_DISCOVER, &dont, sizeof(dont)) < 0) {
+	SU_DEBUG_3(("setsockopt(%s): %s\n",
+		    "IP_MTU_DISCOVER", su_strerror(su_errno())));
+    }
+  }
+#endif
+
 #if HAVE_IP_RECVERR
   if (ai->ai_family == AF_INET || ai->ai_family == AF_INET6) {
-    int const one = 1;
-    if (setsockopt(s, SOL_IP, IP_RECVERR, &one, sizeof(one)) < 0) {
+    if (setsockopt(s, IPPROTO_IP, IP_RECVERR, &one, sizeof(one)) < 0) {
       if (ai->ai_family == AF_INET)
-	SU_DEBUG_3(("setsockopt(IPVRECVERR): %s\n", su_strerror(su_errno())));
+	SU_DEBUG_3(("setsockopt(%s): %s\n",
+		    "IPVRECVERR", su_strerror(su_errno())));
     }
     events |= SU_WAIT_ERR;
   }
 #endif
 #if HAVE_IPV6_RECVERR
   if (ai->ai_family == AF_INET6) {
-    int const one = 1;
-    if (setsockopt(s, SOL_IPV6, IPV6_RECVERR, &one, sizeof(one)) < 0)
+    if (setsockopt(s, IPPROTO_IPV6, IPV6_RECVERR, &one, sizeof(one)) < 0)
       SU_DEBUG_3(("setsockopt(IPV6_RECVERR): %s\n", su_strerror(su_errno())));
     events |= SU_WAIT_ERR;
   }
 #endif
 
-  tl_gets(tags, 
+  tl_gets(tags,
 	  TPTAG_UDP_RMEM_REF(rmem),
 	  TPTAG_UDP_WMEM_REF(wmem),
 	  TAG_END());
 
-  if (rmem != 0 && 
+  if (rmem != 0 &&
+#if HAVE_SO_RCVBUFFORCE
+      setsockopt(s, SOL_SOCKET, SO_RCVBUFFORCE, (void *)&rmem, sizeof rmem) < 0 &&
+#endif
       setsockopt(s, SOL_SOCKET, SO_RCVBUF, (void *)&rmem, sizeof rmem) < 0) {
-    SU_DEBUG_3(("setsockopt(SO_RCVBUF): %s\n", 
+    SU_DEBUG_3(("setsockopt(SO_RCVBUF): %s\n",
 		su_strerror(su_errno())));
   }
 
-  if (wmem != 0 && 
+  if (wmem != 0 &&
+#if HAVE_SO_SNDBUFFORCE
+      setsockopt(s, SOL_SOCKET, SO_SNDBUFFORCE, (void *)&wmem, sizeof wmem) < 0 &&
+#endif
       setsockopt(s, SOL_SOCKET, SO_SNDBUF, (void *)&wmem, sizeof wmem) < 0) {
-    SU_DEBUG_3(("setsockopt(SO_SNDBUF): %s\n", 
+    SU_DEBUG_3(("setsockopt(SO_SNDBUF): %s\n",
 		su_strerror(su_errno())));
   }
 
@@ -157,7 +220,9 @@ int tport_udp_init_primary(tport_primary_t *pri,
 
   tport_check_trunc(pri->pri_primary, ai);
 
+#if HAVE_SOFIA_STUN
   tport_stun_server_add_socket(pri->pri_primary);
+#endif
 
   return 0;
 }
@@ -184,10 +249,13 @@ static void tport_check_trunc(tport_t *tp, su_addrinfo_t *ai)
 
   n = su_sendto(tp->tp_socket,
 		"TEST", 4, 0,
-		(void *)ai->ai_addr, ai->ai_addrlen);
+		(void *)ai->ai_addr, (socklen_t)ai->ai_addrlen);
+
+  if (n != 4)
+    return;
 
   for (;;) {
-    n = su_recvfrom(tp->tp_socket, buffer, sizeof buffer, MSG_TRUNC, 
+    n = su_recvfrom(tp->tp_socket, buffer, sizeof buffer, MSG_TRUNC,
 		    (void *)&su, &sulen);
 
     if (n > (ssize_t)sizeof buffer) {
@@ -205,7 +273,7 @@ static void tport_check_trunc(tport_t *tp, su_addrinfo_t *ai)
 /** Receive datagram.
  *
  * @retval -1 error
- * @retval 0  end-of-stream  
+ * @retval 0  end-of-stream
  * @retval 1  normal receive (should never happen)
  * @retval 2  incomplete recv, call me again (should never happen)
  * @retval 3  STUN keepalive, ignore
@@ -213,7 +281,7 @@ static void tport_check_trunc(tport_t *tp, su_addrinfo_t *ai)
 int tport_recv_dgram(tport_t *self)
 {
   msg_t *msg;
-  ssize_t n, veclen;
+  ssize_t n, veclen, N;
   su_addrinfo_t *ai;
   su_sockaddr_t *from;
   socklen_t fromlen;
@@ -221,17 +289,35 @@ int tport_recv_dgram(tport_t *self)
   uint8_t sample[1];
 
   /* Simulate packet loss */
-  if (self->tp_params->tpp_drop && 
+  if (self->tp_params->tpp_drop &&
       (unsigned)su_randint(0, 1000) < self->tp_params->tpp_drop) {
     su_recv(self->tp_socket, sample, 1, 0);
-    SU_DEBUG_3(("tport(%p): simulated packet loss!\n", self));
+    SU_DEBUG_3(("tport(%p): simulated packet loss!\n", (void *)self));
     return 0;
   }
 
   assert(self->tp_msg == NULL);
 
-  veclen = tport_recv_iovec(self, &self->tp_msg, iovec, 65536, 1);
-  if (veclen < 0)
+#if nomore
+  /* We used to resize the buffer, but it fragments the memory */
+  N = 65535;
+#else
+  N = (ssize_t)su_getmsgsize(self->tp_socket);
+  if (N == -1) {
+    int err = su_errno();
+    SU_DEBUG_1(("%s(%p): su_getmsgsize(): %s (%d)\n", __func__, (void *)self,
+		su_strerror(err), err));
+    return -1;
+  }
+  if (N == 0) {
+    su_recv(self->tp_socket, sample, 1, 0);
+    SU_DEBUG_3(("tport(%p): zero length packet", (void *)self));
+    return 0;
+  }
+#endif
+
+  veclen = tport_recv_iovec(self, &self->tp_msg, iovec, N, 1);
+  if (veclen == -1)
     return -1;
 
   msg = self->tp_msg;
@@ -240,9 +326,9 @@ int tport_recv_dgram(tport_t *self)
   from = (su_sockaddr_t *)ai->ai_addr, fromlen = (socklen_t)(ai->ai_addrlen);
 
   n = su_vrecv(self->tp_socket, iovec, veclen, 0, from, &fromlen);
-  
+
   ai->ai_addrlen = fromlen;
-  
+
   if (n == SOCKET_ERROR) {
     int error = su_errno();
     msg_destroy(msg); self->tp_msg = NULL;
@@ -254,11 +340,14 @@ int tport_recv_dgram(tport_t *self)
       return -1;
   }
   else if (n <= 1) {
-    SU_DEBUG_1(("%s(%p): runt of "MOD_ZD" bytes\n", "tport_recv_dgram", self, n));
+    SU_DEBUG_1(("%s(%p): runt of "MOD_ZD" bytes\n",
+		"tport_recv_dgram", (void *)self, n));
     msg_destroy(msg), self->tp_msg = NULL;
     return 0;
   }
 
+  tport_recv_bytes(self, n, n);
+
   SU_CANONIZE_SOCKADDR(from);
 
   if (self->tp_master->mr_dump_file)
@@ -271,18 +360,20 @@ int tport_recv_dgram(tport_t *self)
 
   if ((sample[0] & 0xf8) == 0xf8)
     /* SigComp */
-    return tport_recv_comp_dgram(self, self->tp_comp, &self->tp_msg, 
+    return tport_recv_comp_dgram(self, self->tp_comp, &self->tp_msg,
 				 from, fromlen);
+#if HAVE_SOFIA_STUN
   else if (sample[0] == 0 || sample[0] == 1)
     /* STUN request or response */
     return tport_recv_stun_dgram(self, &self->tp_msg, from, fromlen);
+#endif
   else
     return 0;
 }
 
 /** Send using su_vsend(). Map IPv4 addresses as IPv6 addresses, if needed. */
-ssize_t tport_send_dgram(tport_t const *self, msg_t *msg, 
-			 msg_iovec_t iov[], 
+ssize_t tport_send_dgram(tport_t const *self, msg_t *msg,
+			 msg_iovec_t iov[],
 			 size_t iovused)
 {
   su_sockaddr_t su[1];
@@ -371,10 +462,10 @@ int tport_udp_error(tport_t const *self, su_sockaddr_t name[1])
   for (c = CMSG_FIRSTHDR(msg); c; c = CMSG_NXTHDR(msg, c)) {
     if (0
 #if HAVE_IP_RECVERR
-	|| (c->cmsg_level == SOL_IP && c->cmsg_type == IP_RECVERR)
+	|| (c->cmsg_level == IPPROTO_IP && c->cmsg_type == IP_RECVERR)
 #endif
 #if HAVE_IPV6_RECVERR
-	|| (c->cmsg_level == SOL_IPV6 && c->cmsg_type == IPV6_RECVERR)
+	|| (c->cmsg_level == IPPROTO_IPV6 && c->cmsg_type == IPV6_RECVERR)
 #endif
 	) {
       char info[128];
@@ -390,12 +481,12 @@ int tport_udp_error(tport_t const *self, su_sockaddr_t name[1])
 	break;
       case SO_EE_ORIGIN_ICMP:
 	origin = "icmp";
-	snprintf(info, sizeof(info), " type=%u code=%u", 
+	snprintf(info, sizeof(info), " type=%u code=%u",
 		 ee->ee_type, ee->ee_code);
 	break;
       case SO_EE_ORIGIN_ICMP6:
 	origin = "icmp6";
-	snprintf(info, sizeof(info), " type=%u code=%u", 
+	snprintf(info, sizeof(info), " type=%u code=%u",
 		ee->ee_type, ee->ee_code);
 	break;
       case SO_EE_ORIGIN_NONE:
@@ -407,16 +498,16 @@ int tport_udp_error(tport_t const *self, su_sockaddr_t name[1])
       }
 
       if (ee->ee_info)
-	snprintf(info + strlen(info), sizeof(info) - strlen(info), 
+	snprintf(info + strlen(info), sizeof(info) - strlen(info),
 		 " info=%08x", ee->ee_info);
 
       SU_DEBUG_3(("%s: %s (%d) [%s%s]\n",
-		  __func__, su_strerror(ee->ee_errno), ee->ee_errno, 
+		  __func__, su_strerror(ee->ee_errno), ee->ee_errno,
 		  origin, info));
       if (from->su_family != AF_UNSPEC)
 	SU_DEBUG_3(("\treported by [%s]:%u\n",
-		    inet_ntop(from->su_family, SU_ADDR(from), 
-			      info, sizeof(info)),
+		    su_inet_ntop(from->su_family, SU_ADDR(from),
+				 info, sizeof(info)),
 		    ntohs(from->su_port)));
 
       if (msg->msg_namelen == 0)
